/**********************************************************************
 *  fujsys.c                                                          *
 *  Assorted system-related functions for Amber 4.1 on Fujitsu        *
 *  VP/VPX/VPP systems.  Some of these are essentially homemade       *
 *  versions of HP Fortran (3F) and Sun FORTRAN functions.            *
 **********************************************************************/

#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/times.h>

void itime_( int *p );
void idate_( int *p );
int access_( char *path, char *mode );
float etime_( float *p );
void exitrcd_( int *n );

/*****  Begin Function "itime_"  **************************************/
/**********************************************************************
 *                                                                    *
 *   ANULIB utility routines for Fujitsu UXP/M    Version 1.3         *
 *                                                                    *
 *   Copyright(c) 1995 The Australian National University             *
 *   This source code is supplied without any warranty, express or    *
 *   implied.  The Australian National University assumes no          *
 *   responsibility for the use or reliability of this software.      *
 *                                                                    *
 **********************************************************************/
/**********************************************************************
 *     file:  fujsys.c
 * function:  itime 
 *  purpose:  FORTRAN callable equivalent of itime() on HP systems
 *  returns:  void
 *   author:  Roger Edberg
 *    dates:  10-Jan-91
 *    notes:  arg is not checked
 **********************************************************************/

void itime_( int *p )
{
   time_t now;
   struct tm *ptm;

   now = time(NULL);
   ptm = localtime(&now);

   *(p  ) = ptm->tm_hour;  /* hour */
   *(p+1) = ptm->tm_min;   /* minute */
   *(p+2) = ptm->tm_sec;   /* second */
   
   return;
}

/**********************************************************************/
/*****  End Function "itime_"  ****************************************/

/*****  Begin Function "idate_"  **************************************/
/**********************************************************************
 *                                                                    *
 *   ANULIB utility routines for Fujitsu UXP/M    Version 1.3         *
 *                                                                    *
 *   Copyright(c) 1995 The Australian National University             *
 *   This source code is supplied without any warranty, express or    *
 *   implied.  The Australian National University assumes no          *
 *   responsibility for the use or reliability of this software.      *
 *                                                                    *
 **********************************************************************/
/**********************************************************************
 *     file:  fujsys.c
 * function:  idate 
 *  purpose:  FORTRAN callable equivalent of idate() on HP systems
 *  returns:  void
 *   author:  Roger Edberg
 *    dates:  10-Jan-91
 *    notes:  arg is not checked
 **********************************************************************/

void idate_( int *p )
{
   time_t now;
   struct tm *ptm;

   now = time(NULL);
   ptm = localtime(&now);

   *(p  ) = ptm->tm_mday;         /* day: 1-31 */
   *(p+1) = ptm->tm_mon + 1;      /* month: 1-12 */
   *(p+2) = ptm->tm_year + 1900;  /* year: 19xx */

   return;
}

/**********************************************************************/
/*****  End Function "idate_"  ****************************************/

/*****  Begin Function "access_"  *************************************/
/**********************************************************************
 *                                                                    *
 *   ANULIB utility routines for Fujitsu UXP/M    Version 1.3         *
 *                                                                    *
 *   Copyright(c) 1995 The Australian National University             *
 *   This source code is supplied without any warranty, express or    *
 *   implied.  The Australian National University assumes no          *
 *   responsibility for the use or reliability of this software.      *
 *                                                                    *
 **********************************************************************/
/**********************************************************************
 *     file:  fujsys.c
 * function:  access_
 *  purpose:  FORTRAN callable equivalent of access(3F) on HP systems
 *  returns:  int
 *   author:  Roger Edberg
 *    dates:  10-Jan-95
 *    notes:  
 **********************************************************************/

int access_( char *path, char *mode )
{
   int ierr, imode;
   int i_r, i_w, i_x, i_e;
  
   i_r = i_w = i_x = i_e = 0;
      
   if ( strstr(mode,"r") != NULL ) i_r = R_OK; 
   if ( strstr(mode,"w") != NULL ) i_w = W_OK; 
   if ( strstr(mode,"x") != NULL ) i_x = X_OK; 
   if ( strstr(mode," ") != NULL ) i_e = F_OK; 

   imode = i_r | i_w | i_x | i_e ;
   ierr = access(path,imode);
   return ierr;
}

/**********************************************************************/
/*****  End Function "access_"  ***************************************/

/*****  Begin Function "etime"  **************************************/
/**********************************************************************
 *                                                                    *
 *   ANULIB utility routines for Fujitsu UXP/M    Version 1.3         *
 *                                                                    *
 *   Copyright(c) 1992 The Australian National University             *
 *   This source code is supplied without any warranty, express or    *
 *   implied.  The Australian National University assumes no          *
 *   responsibility for the use or reliability of this software.      *
 *                                                                    *
 **********************************************************************/
/**********************************************************************
 *     file:  fujsys.c
 * function:  etime
 *  purpose:  equivalent of etime() in Sun Fortran
 *  returns:  float (seconds)
 *   author:  Roger Edberg
 *    dates:  14-Nov-91, revised 3-May-93,11-Jun-93,10-May-95
 *    notes:  -maybe should use timesu(2) call instead if times(2)
 *            -does timesu() return vector unit time info?
 *            -modified to [1] use sysconf() [POSIX] instead of compile 
 *             time const CLK_TCK, [2] use automatic variable ts rather
 *             than malloc/free ts each time etime is called.
 **********************************************************************/

float etime_( float *p )
{
   struct tms ts;
   long clk_tck;
    
   if ( times(&ts) < 0 ) {
      fprintf(stderr, "etime_: times returned error\n");
      return -1;
   }
    
   if ( (clk_tck = sysconf(_SC_CLK_TCK)) < 0 ) {
      fprintf(stderr, "etime_: sysconf returned error\n");
      return -1;
   }

   *(p  ) = (float) ( ts.tms_utime + ts.tms_cutime ) / clk_tck;
   *(p+1) = (float) ( ts.tms_stime + ts.tms_cstime ) / clk_tck;

   return (*p + *(p+1));
}

/**********************************************************************/
/*****  End of Function "etime"  *************************************/

/*****  Begin Function "exitrcd_"  ************************************/
/**********************************************************************
 *                                                                    *
 *   ANULIB utility routines for Fujitsu UXP/M    Version 1.3         *
 *                                                                    *
 *   Copyright(c) 1995 The Australian National University             *
 *   This source code is supplied without any warranty, express or    *
 *   implied.  The Australian National University assumes no          *
 *   responsibility for the use or reliability of this software.      *
 *                                                                    *
 **********************************************************************/
/**********************************************************************
 *     file:  fujsys.c
 * function:  exitrcd_
 *  purpose:  FORTRAN callable exit that rtns proper code to shell
 *  returns:  -doesn't-
 *   author:  Roger Edberg
 *    dates:  10-Jan-91
 *    notes:  part of hack around unpredictable setrcd() on the VPP
 **********************************************************************/

void exitrcd_( int *n )
{
   _exit(*n);
}

/**********************************************************************/
/*****  End Function "exitrcd_"  **************************************/


