      subroutine MPI_reduce_scatter
     &                      ( sendbuf, recvbuf, count,
     &                        MPI_dat, MPI_op, MPI_comm, ierr )

      implicit none

#include "mpif.h"

      integer count(*), MPI_dat, MPI_op
      integer MPI_comm,ierr
      double precision sendbuf(*), recvbuf(*)

      integer myid, numnodes

      integer Nele, Pmax
      parameter(Nele=500000,Pmax=128)

      integer ncall, r1, r2, rm, is, N, N0, p, p2, i, id
      integer nsucc
      integer status ( MPI_STATUS_SIZE )

      integer adr(0:Pmax)

      integer last_comm
      save last_comm, ncall, myid, numnodes, p, p2

      data ncall /-1/

      integer TAG
      parameter(TAG=1)

      double precision internal_buf(Nele)

      character*4 fil
      save fil


      if( MPI_dat.ne.MPI_DOUBLE_PRECISION ) then
        print*, 
     &'Err SMPI_REDUCE_SCATTER: only MPI_DOUBLE_PRECISION supported '
        ierr = 1
        return
      endif

      if( MPI_op.ne.MPI_SUM ) then
        print*, 
     &'Err SMPI_REDUCE_SCATTER: only MPI_SUM supported '
        ierr = 1
        return
      endif

      if( ncall.eq.-1 .or. MPI_comm.ne.last_comm ) then


      call MPI_COMM_RANK( MPI_comm, myid, ierr )
      call MPI_COMM_SIZE( MPI_comm, numnodes, ierr )

      if( ierr.ne.0 ) return

      endif

c     if( ncall.eq.-1 ) then
c     write(fil,'(a2,i2.2)') 't_',myid
c     open(20,file=fil)
c     endif

      ncall = 0
      last_comm = MPI_comm

      adr(0) = 1
      do i=1,numnodes
       adr(i) = adr(i-1)+count(i)
      enddo

      do i=numnodes+1,numnodes+numnodes
       adr(i) = adr(numnodes)
      enddo

      N0 = adr(numnodes)

      if( Nele.lt.2*N0 ) then
        print*, 'Err SMPI_REDUCE_SCATTER: internal buffer overflow '
        ierr = 1
        return
      endif

c     write(20,*) 'Adrs: ',(adr(i),i=0,numnodes)

      do N=1,adr(numnodes)-1
        internal_buf(N) = sendbuf(N)
      enddo

c Communications pattern:


      r1 = 0
      r2 = numnodes-1

      do while( r1.lt.r2 )

        p2 =(r2-r1)/2 + 1
        rm = r1+p2

        if( myid.lt.rm ) then

          if( myid+p2.le.r2 ) then
c          write(20,*) 'Partner: ',myid+p2,' Adr: ',adr(rm),adr(r2+1)-1
c          write(20,7) 'Data: ',(internal_buf(i),i=adr(rm),adr(r2+1)-1)
c          write(20,*) 'Recv: ',adr(r1),adr(rm)-1

           call MPI_SENDRECV(
     &              internal_buf(adr(rm)),adr(r2+1)-adr(rm),
     &              MPI_DOUBLE_PRECISION,
     &              myid+p2,TAG,
     &              internal_buf(N0),adr(rm)-adr(r1),
     &              MPI_DOUBLE_PRECISION,
     &              myid+p2,TAG,
     &              MPI_COMM_WORLD, status, ierr )

           do N=0,adr(rm)-adr(r1)-1
              internal_buf(adr(r1)+N) = 
     &           internal_buf(adr(r1)+N) + internal_buf(N0+N)
           enddo
c          write(20,7) 
c    &      'Recvd: ',(internal_buf(i),i=N0,N0+adr(rm)-adr(r1)-1)


          else if( r2.ge.rm ) then

c          write(20,*) 'Send -Partner: ',r2,' Adr: ',adr(rm),adr(r2+1)-1
c          write(20,7) 'Data: ',(internal_buf(i),i=adr(rm),adr(r2+1)-1)
            call MPI_SEND
     &             (internal_buf(adr(rm)),adr(r2+1)-adr(rm),
     &              MPI_DOUBLE_PRECISION,
     &              r2,TAG,
     &              MPI_COMM_WORLD, ierr )

          endif
 
          r2 = min(r2,rm-1)

        else

c         write(20,*) 'Partner: ',myid-p2,' Adr: ',adr(r1),adr(rm)-1
c          write(20,7) 'Data: ',(internal_buf(i),i=adr(r1),adr(rm)-1)
c          write(20,*) 'Recv: ',adr(rm),adr(r2+1)-1

           call MPI_SENDRECV(
     &              internal_buf(adr(r1)),adr(rm)-adr(r1),
     &              MPI_DOUBLE_PRECISION,
     &              myid-p2,TAG,
     &              internal_buf(N0),adr(r2+1)-adr(rm),
     &              MPI_DOUBLE_PRECISION,
     &              myid-p2,TAG,
     &              MPI_COMM_WORLD, status, ierr )

           do N=0,adr(r2+1)-adr(rm)-1
              internal_buf(adr(rm)+N) = 
     &           internal_buf(adr(rm)+N) + internal_buf(N0+N)
           enddo
c          write(20,7) 
c    &      'Recvd: ',(internal_buf(i),i=N0,N0+adr(r2+1)-adr(rm)-1)


          if( myid.eq.r2 ) then
          do id=r2+1,rm+p2-1
c           write(20,*) 'Recv-Partner: ',id-p2,
c    &                  ' Adr: ',adr(rm),adr(r2+1)-1
            call MPI_RECV
     &             (internal_buf(N0),adr(r2+1)-adr(rm),
     &              MPI_DOUBLE_PRECISION,
     &              id-p2,TAG,
     &              MPI_COMM_WORLD, status, ierr )

           do N=0,adr(r2+1)-adr(rm)-1
              internal_buf(adr(rm)+N) = 
     &           internal_buf(adr(rm)+N) + internal_buf(N0+N)
           enddo
c          write(20,7) 
c    &      'Recvd: ',(internal_buf(i),i=N0,N0+adr(r2+1)-adr(rm)-1)

          enddo

          endif


          r1 = rm

        endif

        

      enddo


      do N=1,count(myid+1)

        recvbuf(N) = internal_buf(adr(myid)-1+N)

      enddo

c     write(20,7) ' Res: ',(recvbuf(i),i=1,count(myid+1))


 7    format(a,/(6f8.2))

      return

      end
