c*****************************************************************
c
c ----------------------------------------------------------------
c IBM SP1 implementation of MPI routines for message passing AMBER
c ----------------------------------------------------------------
c
c     The original implementation was written and contributed by:
c
c		James J. Vincent
c		Kenneth M. Merz
c		Department of Chemistry
c		July, 1994
c
c	      ***********************   
c	      *                     *   
c	      * Pennsylvania  State *   
c	      *      University     *   
c	      *                     *   
c	      *                     *   
c	      *    1   8   5   5    *   
c	      *                     *   
c	       **                 **    
c	         ***           ***     
c	            ****   ****
c	                ***
c
c     COPYRIGHT 1994, Pennsylvania State University.
c                 All rights reserved.  
c
c     This software is covered by the AMBER license 
c     agreement which contains restrictions on its use, 
c     disclosure, and duplication!
c
c   Usage notes:  
c
c     Include -DMPI -DPOE in MACHINE file for use on IBM SP1
c     with high speed switch.
c     
c***********************************************************

      subroutine MPI_BARRIER(MPI_COMM_WORLD,ierr)
c***********************************************************
c     Synchronize tasks with a call to barrier.
c**********************************************************
      common/locpoe/allgrp
      integer allgrp
     
      integer ierr	 

      call mp_sync(allgrp)
      return
      end

      subroutine MPI_ABORT(ierr)
c************************************************************
c     MP_STOPALL kills all tasks and stops program.
c************************************************************
      integer ierr

      call mp_stopall(0)
      return
      end

      subroutine MPI_FINALIZE(ierr)
c************************************************************
c     ierr is supposed to be a returned condition
c     but here is used as an input exit code
c************************************************************
      integer ierr

      return
      end
      

      subroutine MPI_INIT(ierr)
c*************************************************************
c     Enrolls program 
c     Task PE number and total # tasks set
c*************************************************************
#include "parallel.h"
      common/locpoe/allgrp
      integer allgrp
      
      integer nbuf
      dimension nbuf(4)

c     Enroll this program in POE
      call mp_environ(numtasks,mytaskid)
      call mp_task_query(nbuf,4,3)
      allgrp = nbuf(4)

      return
      end

      subroutine MPI_COMM_RANK(MPI_COMM_WORLD,rank,ierr)
c************************************************************
c     Get my PE number (rank) 
c************************************************************
#include "parallel.h"
      integer rank,ierr

      rank = mytaskid
      return
      end

      subroutine MPI_COMM_SIZE(MPI_COMM_WORLD,size,ierr)
c************************************************************
c     Get total number of tasks (size) 
c************************************************************
#include "parallel.h"
      integer size,ierr

      size = numtasks
      return
      end

      subroutine MPI_RECV(msg,count,datatype,source,tag,comm,ist,ierr)
c***********************************************************
c     Receive a message 
c     msg: buffer to put message in
c     count : number of datatype elements
c     datatype: type of data
c     source: PE number of sender
c     tag: message tag
c     comm: group comm id
c     ist: returned status of message receive-not used
c     ierr: returned error condition - not used
c**********************************************************
#include "parallel.h"
#include "mpif.h"      
c     *** Arguments 
      dimension msg(*)
      real msg
      integer count,datatype,source,tag,comm,ierr

c     Local
      integer size

      if(datatype .eq.  MPI_INTEGER2)size=count*2
      if(datatype .eq.  MPI_INTEGER) size=count*4
      if(datatype .eq.  MPI_REAL)    size=count*4
      if(datatype .eq.  MPI_DOUBLE_PRECISION)size=count*8

c     RECV message
c     ierr actually returns number of bytes received
      call mp_brecv(msg,size,source,tag,ierr)
      return
      end

      subroutine MPI_SEND(msg,count,datatype,dest,tag,comm,ierr)
c***********************************************************
c     Send a message 
c     msg: buffer message is in
c     size: size of message IN BYTES
c     dest: PE number of destination process (0->n-1)
c     type: message type
c     comm: MPI communicator handdle - not used
c     ierr: returned error condition - not used
c**********************************************************
#include "parallel.h"
#include "mpif.h"      
c     *** Arguments
      dimension msg(*)
      real msg
      integer count,datatype,dest,tag,comm,ierr

c     *** Local 
      integer size

      if(datatype .eq.  MPI_INTEGER2)size=count*2
      if(datatype .eq.  MPI_INTEGER) size=count*4
      if(datatype .eq.  MPI_REAL)    size=count*4
      if(datatype .eq.  MPI_DOUBLE_PRECISION)  size=count*8

      call mp_send(msg,size,dest,tag,ierr)
      return
      end


      subroutine MPI_BCAST(msg,count,datatype,root,comm,ierr)
c************************************************************
c     Broadcast the array 'msg' to all nodes in the configuration
c     (should be all nodes in group really)
c     msg type can be anything
c     datatype: type of data being broadcast
c     root: the node which will do the broadcasting
c     group: integer identifier of group to broadcast to -not used
c     count: Number of datatype elements in msg
c************************************************************
#include "parallel.h"
#include "mpif.h"      
      common/locpoe/allgrp
      integer allgrp

      integer count,root,comm,i,ierr,datatype
      dimension msg(*)
      real msg
c     *** Local
      integer size

      if(datatype .eq.  MPI_INTEGER2)size=count*2
      if(datatype .eq.  MPI_INTEGER) size=count*4
      if(datatype .eq.  MPI_REAL)    size=count*4
      if(datatype .eq.  MPI_DOUBLE_PRECISION)  size=count*8

      call mp_bcast(msg,size,root,allgrp)
      return 
      end
      
      subroutine MPI_REDUCE(from,to,count,datatype,op,root,comm,ierr) 
c******************************************************************
c     Perform reduction operation on the array 'from' across all
c     nodes and place result in 'to' array. Reduction operation is
c     double precision add only. Result is placed on node that did
c     spawning (master) at startup.
c     Could generalize to put result on any node using taskids
c*****************************************************************
#include "parallel.h"
#include "mpif.h"      
      common/locpoe/allgrp
      integer allgrp
      external d_vadd
      integer count,datatype,op,root,comm,ierr

c     Input, output arrays
      dimension from(*),to(*)
      real from,to
c     *** Local 
      integer size

      if(datatype .eq.  MPI_INTEGER2)size=count*2
      if(datatype .eq.  MPI_INTEGER) size=count*4
      if(datatype .eq.  MPI_REAL)    size=count*4
      if(datatype .eq.  MPI_DOUBLE_PRECISION)  size=count*8

      call mp_reduce(from,to,size,root,d_vadd,allgrp)

      return
      end

      subroutine MPI_ALLREDUCE(from,to,count,datatype,op,comm,ierr)
c************************************************************
c     Perform reduction operation on the array 'from' from all
c     nodes then broadcast the resulting array back to all nodes
c     and place on 'to'. 
c     ONLY DOUBLE ADD IS PERFORMED.
c************************************************************
#include "parallel.h"
#include "mpif.h"      
      common/locpoe/allgrp

      external d_vadd
      integer count,datatype,op,comm,ierr
      real from,to
      dimension from(*),to(*)
c     *** Local 
      integer size

      if(datatype .eq.  MPI_INTEGER2)size=count*2
      if(datatype .eq.  MPI_INTEGER) size=count*4
      if(datatype .eq.  MPI_REAL)    size=count*4
      if(datatype .eq.  MPI_DOUBLE_PRECISION)  size=count*8

      call mp_combine(from,to,size,d_vadd,allgrp)  

      return
      end

      subroutine MPI_CONCAT(outmsg,inmsg,count,datatype,comm,ierr)
c*********************************************************
c     !!!!ONLY DOES INTEGER!!!!(for now)
c     Concatenate outmsg from each process into inmsg by
c     order of PE #. Inmsg is (numtasks)*outmsg in length
c     inmsg: local message to get concatenated
c     inmsg: copy of all messages, placed back on all tasks
c     size: size of OUTMSG in BYTES
c     group: integer identifying group of tasks (not used)
c     !!!!outmsg gets destroyed in the process!!!
c************************************************************
#include "parallel.h"
#include "mpif.h"      
      common/locpoe/allgrp
      integer allgrp

c     *** Arguments
      dimension outmsg(*),inmsg(*)
      integer outmsg,inmsg

      integer count,datatype,comm,ierr

c     *** Local 
      integer i

      call MPI_BARRIER(MPI_COMM_WORLD,1)

c     Everyone send their data to 0 (including 0)
      call MPI_SEND(outmsg,count,datatype,0,102,comm,ierr)
c     PE # 0 does the work
      if(mytaskid .eq. 0) then      
c     Make sure to recv messages in order
         do i = 0,numtasks-1
          call MPI_RECV(inmsg(i*count+1),count,datatype,
     +                  i,102,comm,ist,ierr)
         enddo
      endif
c     Broadcast concatenated msg back to all
      call MPI_BCAST(inmsg,count*numtasks,datatype,0,comm,ierr)
      return
      end


c
c     The IBM SP2, to my knowledge, does not have collective
c     communication routines which allow each processor to
c     "send" different size chunks all at once, as can be 
c     done in an MPI_ALLGATHERV.  Therefore, this routine is 
c     implemented as a series of broadcasts.  This is accomplished
c     by having the originating processor copy the correct values
c     into the recvbuf() array then broadcasting the values.
c     This will obviously be more efficient if we perform data
c     in place mpi operations.  In order to handle typing in the
c     array copy (without using bcopy()), we call a different
c     routine (MPI_ALLGATHERV_DOUBLE() to process double precision
c     values. (tec3)
c
      subroutine MPI_ALLGATHERV(sendbuf, sendcount, sendtype,
     .   recvbuf, recvcounts, displs, recvtype, comm,
     .   ierror)
      real sendbuf(*), recvbuf(*)
      integer sendcount, sendtype, recvcounts(*), displs(*),
     .   recvtype, comm, ierror

#include "mpif.h"
#include "parallel.h"

      common/locpoe/allgrp
      integer allgrp

      integer size


      if(sendtype .eq. MPI_DOUBLE_PRECISION) then
         call MPI_ALLGATHERV_DOUBLE(sendbuf, sendcount, sendtype,
     .   recvbuf, recvcounts, displs, recvtype, comm,
     .   ierror)
         return
      else if (sendtype .eq. MPI_INTEGER2) then
         write (6,*) 'MPI_ALLGATHERV in IBM POE/MPL AMBER/MPI'
         write (6,*) 'implementation does not currently handle'
         write (6,*) 'MPI_INTEGER2.  Exitting...'
         call MPI_ABORT(1)
      endif

      do i = 0, numtasks-1
         size = recvcounts(i+1)
         if (i .eq. mytaskid) then
            do j = 1, size
               recvbuf(displs(i+1)+j) = sendbuf(j)
            enddo
         endif

         if(sendtype .eq. MPI_INTEGER) size=size*4
         if(sendtype .eq. MPI_REAL)    size=size*4

         call mp_bcast(recvbuf(displs(i+1)+1),size,i,allgrp)

      enddo

      return
      end

      subroutine MPI_ALLGATHERV_DOUBLE(sendbuf, sendcount, sendtype,
     .   recvbuf, recvcounts, displs, recvtype, comm,
     .   ierror)
      double precision sendbuf(*), recvbuf(*)
      integer sendcount, sendtype, recvcounts(*), displs(*),
     .   recvtype, comm, ierror

#include "mpif.h"
#include "parallel.h"

      common/locpoe/allgrp
      integer allgrp

      integer size

      do i = 0, numtasks-1
         size = recvcounts(i+1)
         if (i .eq. mytaskid) then
            do j = 1, size
               recvbuf(displs(i+1)+j) = sendbuf(j)
            enddo
         endif
         if(sendtype .eq. MPI_DOUBLE_PRECISION) size=size*8

         call mp_bcast(recvbuf(displs(i+1)+1),size,i,allgrp)

      enddo

      return
      end
    


    

       


