c************************************************************
c
c     This is a template of the necessary MPI routines and
c     arguments for the AMBER MPI parallel code. 
c
c     The original MPI code was written and contributed by:
c
c     James J. Vincent
c     Kenneth M. Merz
c     Department of Chemistry
c     Pennsylvania State University
c     July, 1994
c
c     The following routines must be specified for proper functioning
c     of the MPI parallel AMBER:
c
c     MPI_REDUCE
c     MPI_BCAST
c     MPI_INIT
c     MPI_FINALIZE
c     MPI_RANK
c     MPI_SIZE
c     MPI_BARRIER
c
c***********************************************************

      subroutine MPI_BARRIER(group)
c***********************************************************
c     Synchronize all tasks
**********************************************************
#include "mpif.h"      
      integer group

      return
      end

      subroutine MPI_FINALIZE(ierr)
c************************************************************
c     Execute any exit code specific to parallel machine
c************************************************************
#include "mpif.h"
      integer ierr

      end
      
      subroutine MPI_INIT(ierr)
c*************************************************************
c     Enrolls caller in parallel machine.
c     Must set number of tasks and own PE number.
c*************************************************************
#include "mpif.h"
      integer ierr


      return
      end

      subroutine MPI_COMM_RANK(MPI_COMM_WORLD,rank,ierr)
c************************************************************
c     Get my PE number (rank) 
c************************************************************
#include "mpif.h"
      integer rank,ierr

      rank = mytaskid

      return
      end


      subroutine MPI_COMM_SIZE(MPI_COMM_WORLD,size,ierr)
c************************************************************
c     Get size of machine (number of PE's)
c************************************************************
#include "mpif.h"
      integer size,ierr

      size = numtasks
      return
      end

      subroutine MPI_RECV(msg,count,datatype,source,tag,comm,ierr)
c***********************************************************
c     Receive a message 
c     msg: buffer to put message in
c     count : number of datatype elements
c     datatype: type of data
c     source: PE number of sender
c     tag: message tag
c     comm: group comm id (not used)
c     status: returned status of message receive
c**********************************************************
#include "mpif.h"
c     *** Arguments 
      dimension msg(*)
      double precision msg
      integer count,datatype,source,tag,comm,ierr

      return
      end

      subroutine MPI_SEND(msg,count,datatype,dest,tag,comm,ierr)
c***********************************************************
c     Send a message 
c     msg: buffer message is in
c     count: number of datatype elements in message
c     datatype MPI datatype of message elements
c     dest: PE number of destination process (0->n-1)
c     tag: message type
c     comm: group identifier for communication (not used yet)
c**********************************************************
#include "mpif.h"
      dimension msg(*)
      double precision msg

      return
      end

      subroutine MPI_BCAST(msg,count,datatype,root,comm,ierr)
c************************************************************
c     Broadcast the array 'msg' to all nodes in the configuration
c     msg type can be anything
c     datatype: type of data being broadcast
c     root: the node which will do the broadcasting
c     group: integer identifier of group to broadcast to -not used
c     count: Number of datatype elements in msg
c************************************************************
#include "mpif.h"

      integer count,datatype,root,comm,ierr,stype
      
      return 
      end
      

      subroutine MPI_REDUCE(from,to,count,datatype,op,root,comm,ierr)
c******************************************************************
c     Perform reduction operation on the array 'from' across all
c     nodes and place result in 'to' array.
c*****************************************************************
#include "mpif.h"

c     *** Arguments ****
      integer count,datatype,root,comm,ierr
c     This nodes portion of the array to reduce
      dimension from(*)
      double precision from
c     Array to store result in
      dimension to(*)
      double precision to

      return
      end

      subroutine MPI_COMBINE(from,to,count,datatype,
     &                                   op,comm,ierr)
c************************************************************
c     Perform reduction operation on the array 'from' from all
c     nodes then broadcast the resulting array back to all nodes
c     and place on 'to'. 
c************************************************************
#include "mpif.h"

c     *** Arguments
      dimension from(*),to(*)
      double precision to,from
      integer count,datatype,comm,ierr
      external op

c     Generic implementation - can be used if REDUCE and BCAST
c     are implemented on any architecture.
c     Use node 0 as the central gatherer
      call MPI_REDUCE(from,to,count,datatype,op,0,comm,ierr)
      call MPI_BCAST(to,count,datatype,0,comm,ierr)
 
      return
      end

      subroutine MPI_CONCAT(outmsg,inmsg,count,datatype,comm,ierr)
c*********************************************************
c     Concatenate inmsg from each process into outmsg by
c     order of PE #. outmsg is (nhosts)*inmsg in length
c     inmsg: local message to get concatenated
c     outmsg: copy of all messages, placed back on all tasks
c     size: size of OUTMSG in BYTES
c     group: integer identifying group of tasks (not used)
c************************************************************
#include "mpif.h"

c     *** Arguments
      dimension outmsg(*),inmsg(*)
      integer outmsg,inmsg
      integer count,datatype,comm,ierr

      return
      end

