c-----------------------------------------------------------------------
      subroutine bonerr(nbond,ibond,jbond,x,y,z,name,lun)
C
C************************************************************************
C                              AMBER                                   **
C                                                                      **
C                  Copyright (c) 1986, 1991, 1995                      **
C             Regents of the University of California                  **
C                       All Rights Reserved.                           ** 
C                                                                      **
C  This software provided pursuant to a license agreement containing   **
C  restrictions on its disclosure, duplication, and use. This software **
C  contains confidential and proprietary information, and may not be   **
C  extracted or distributed, in whole or in part, for any purpose      **
C  whatsoever, without the express written permission of the authors.  **
C  This notice, and the associated author list, must be attached to    **
C  all copies, or extracts, of this software. Any additional           **
C  restrictions set forth in the license agreement also apply to this  **
C  software.                                                           **
C************************************************************************
C
c     finds and prints sorted list of 10 longest bonds
c     Author:  George Seibel
c
*     implicit none
c
c     PARAMETERS:
c
      integer MAXLNG
      parameter (MAXLNG=5)
c        ... max number of long bonds to print
c
c     INPUT:
c
      integer nbond
c        ... number of bonds
      integer ibond(*), jbond(*)
c        ... pointers to i,j atoms of bond
      real x(*), y(*), z(*)
c        ... coordinates
      integer name(*)
c        ... Hollerith atom names
      integer lun
c        ... logical unit number for output
c
c     INTERNAL:
c
      integer nlong
c        ... number of long bonds, max is MAXLNG
      integer long(MAXLNG)
c        ... pointers to ibond(),jbond() for long bonds
      real rlong(MAXLNG)
c        ... lengths of long bonds
      integer low
c        ... pointer to lowest value of rlong()
      integer ib,jb
c        ... temps for bond pointers
      real r
c        ... temp for distances
      real eps
c        ... a sensible epsilon, sufficient for all machines
      logical repeat
c        ... repeat flag for bubble sort
      real rswap
      integer iswap
c        ... swap temps for bubble sort
      integer i
c        ... loop index
c
      data eps /1.0e-5/
c
c     --- load long bond list ---
c
      nlong = min(MAXLNG,nbond)
      do 100 i = 1, nlong
           ib = ibond(i)
           jb = jbond(i)
           r = sqrt((x(ib)-x(jb))**2+(y(ib)-y(jb))**2+(z(ib)-z(jb))**2)
           long(i) = i
           rlong(i) = r
  100 continue
c
c     --- find lowest value of rlong() ---
c
      call getlow(nlong,rlong,low)
c
c     --- loop through remaining bonds, replacing lowest of longs ---
c
      do 200 i = nlong+1, nbond
           ib = ibond(i)
           jb = jbond(i)
           r = sqrt((x(ib)-x(jb))**2+(y(ib)-y(jb))**2+(z(ib)-z(jb))**2)
           if (r - rlong(low) .gt. eps) then
                long(low) = i
                rlong(low) = r
                call getlow(nlong,rlong,low)
           endif
  200 continue
c
c     --- sort the long bond table in descending order ---
c
  300 continue
           repeat = .false.
           do 400 i = 1, nlong -1
                if (rlong(i) .lt. rlong(i+1)) then
                     repeat = .true.
                     rswap = rlong(i)
                     rlong(i) = rlong(i+1)
                     rlong(i+1) = rswap
                     iswap = long(i)
                     long(i) = long(i+1)
                     long(i+1) = iswap
                endif
  400      continue
      if (repeat) go to 300
c
c     --- print the long bonds ---
c
      write(lun,'(/5x,i3,a,/)') nlong,' Longest Bonds in the System:'
      do 500 i = 1, nlong
           ib = ibond(long(i))
           jb = jbond(long(i))
           write(lun,'(5x,a4,i5,2x,a4,i5,f14.4)')
     +           name(ib),ib,name(jb),jb,rlong(i)
  500 continue
      return
      end
