

c-----------------------------------------------------------------------
      subroutine pdbrd(lunpdb,lunerr,iatnum,iatnam,irsnam,irsnum,x,y,z,
     +                 endfil)
C
C************************************************************************
C                              AMBER                                   **
C                                                                      **
C                  Copyright (c) 1986, 1991, 1995                      **
C             Regents of the University of California                  **
C                       All Rights Reserved.                           ** 
C                                                                      **
C  This software provided pursuant to a license agreement containing   **
C  restrictions on its disclosure, duplication, and use. This software **
C  contains confidential and proprietary information, and may not be   **
C  extracted or distributed, in whole or in part, for any purpose      **
C  whatsoever, without the express written permission of the authors.  **
C  This notice, and the associated author list, must be attached to    **
C  all copies, or extracts, of this software. Any additional           **
C  restrictions set forth in the license agreement also apply to this  **
C  software.                                                           **
C************************************************************************
C
c     Reads lines from a pdb file until an 'ATOM' or 'HETATM' record
c     is encountered, then loads the ever-so-fine Amber data structure
c     from that record.  Handles true Brookhaven PDB records, and atom
c     names are always left-justified in iatnam regardless of their
c     justification in col. 13 or 14.  Returns .true. in endfil at EOF.
c
c     A funny Brookhaven convention is used for some hydrogens: if
c     they are pro-chiral, the final number that the IUPAC standard
c     requires is placed at the beginning of the name (i.e. in column
c     13.  This routine turns things around.  --Further--, what 
c     Brookhaven thinks of as IUPAC numbering does not agree with
c     what most of the nmr community (and the db4.dat database) think
c     of as the correct numbering.  So, inconsistencies there are
c     fixed as well.     (These additions by dac, 10/90)
c
c     Author:  George Seibel
*     implicit none
c
c     INPUT:
c
      integer lunpdb
c        ... logical unit number of open PDB file to be read
      integer lunerr
c        ... logical unit number of error output channel
c
c     OUTPUT:
c
      integer iatnum
c        ... atom serial number from pdb file
      integer iatnam
c        ... atom name
      integer irsnam
c        ... residue name
      integer irsnum
c        ... residue number
      real x, y, z
c        ... coordinates
      logical endfil
c        ... returned .true. on EOF
c
c     INTERNAL:
c
      character*80 buf
c        ... buffer for input line
      character*4 cat
c        ... temporary string for revised atom name
c
      endfil = .false.
c
c     --- read lines until 'ATOM' or 'HETATM' record, or until EOF ---
c   
  100 continue
           read(lunpdb,'(a)',end=1000) buf
      if (buf(1:4) .ne. 'ATOM' .and. buf(1:4) .ne. 'HETA') go to 100
c
c     --- skip atoms that represent alternate conformers
c
      if (buf(17:17) .ne. ' ' .and. buf(17:17) .ne. 'A') go to 100
c
c     --- test for column number of start of atom name ---
c
      if (buf(13:13) .eq. ' ') then
c          --- heavy-atom name, starts in col. 14 ---
           read(buf,9000,err=2000) iatnum,iatnam,irsnam,irsnum,x,y,z
      else
c          --- H/D name, starts in col. 13 ---
           read(buf,9010,err=2000) iatnum,iatnam,irsnam,irsnum,x,y,z
      endif
c
c  --- now fix up atom name for hydrogens, as discussed above
c
      if (buf(13:13).eq.'1' .or. buf(13:13).eq.'2' 
     +    .or. buf(13:13).eq.'3') then
        cat(1:1) = buf(14:14)
        cat(2:2) = buf(15:15)
        cat(3:3) = buf(16:16)
        if (cat(3:3).eq.' ') then
          cat(3:3) = buf(13:13)
          cat(4:4) = ' '
        else
          cat(4:4) = buf(13:13)
        end if
        read (cat,'(a4)') iatnam
      end if
      return
c
c     --- hit EOF ---
c
 1000 continue
      endfil = .true.
      return
c
c     --- error reading PDB record ---
c
 2000 continue
      write(lunerr,'(/2x,a,/,a)') 'Error reading PDB line:',buf
      call mexit(lunerr, 1)
 9000 format(6x,i5,2x,a3,1x,a3,2x,i4,4x,3f8.3)
 9010 format(6x,i5,1x,a4,1x,a3,2x,i4,4x,3f8.3)
      end     
