c-----------------------------------------------------------------------
      block data raset
C
C************************************************************************
C                              AMBER                                   **
C                                                                      **
C                  Copyright (c) 1986, 1991, 1995                      **
C             Regents of the University of California                  **
C                       All Rights Reserved.                           ** 
C                                                                      **
C  This software provided pursuant to a license agreement containing   **
C  restrictions on its disclosure, duplication, and use. This software **
C  contains confidential and proprietary information, and may not be   **
C  extracted or distributed, in whole or in part, for any purpose      **
C  whatsoever, without the express written permission of the authors.  **
C  This notice, and the associated author list, must be attached to    **
C  all copies, or extracts, of this software. Any additional           **
C  restrictions set forth in the license agreement also apply to this  **
C  software.                                                           **
C************************************************************************
C
c     Initializes random number initialization flag to false.
      double precision u(97), c, cd, cm
c        ... real variables in Marsaglia algorithm
      integer i97, j97
c        ... pointers into u() in Marsaglia algorithm
      logical set
c        ... true if amrset has been called
      common /raset1/ u, c, cd, cm, i97, j97, set
c
      data set /.false./
      end
c-----------------------------------------------------------------------
      subroutine amrset(iseed)
C
C************************************************************************
C                              AMBER                                   **
C                                                                      **
C                  Copyright (c) 1986, 1991, 1995                      **
C             Regents of the University of California                  **
C                       All Rights Reserved.                           ** 
C                                                                      **
C  This software provided pursuant to a license agreement containing   **
C  restrictions on its disclosure, duplication, and use. This software **
C  contains confidential and proprietary information, and may not be   **
C  extracted or distributed, in whole or in part, for any purpose      **
C  whatsoever, without the express written permission of the authors.  **
C  This notice, and the associated author list, must be attached to    **
C  all copies, or extracts, of this software. Any additional           **
C  restrictions set forth in the license agreement also apply to this  **
C  software.                                                           **
c     Initialization routine for Marsaglias random number generator
c     as implemented in Amber 3.0 Rev A by George Seibel
c     See doc in amrand.
c     Testing:  Call amrset with iseed = 54185253.  This should result
c     in is1 = 1802 and is2 = 9373.  Call amrand 20000 times, then six
c     more times, printing the six random numbers * 2**24 (ie, 4096*4096)
c     They should be: (6f12.1)
c     6533892.0  14220222.0  7275067.0  6172232.0  8354498.0  10633180.0
c
c     INPUT:
c
      integer iseed
c        ... integer seed greater than zero
c
c     OUTPUT: (to common/raset1/)
c
      double precision u(97), c, cd, cm
c        ... real variables in Marsaglia algorithm
      integer i97, j97
c        ... pointers into u() in Marsaglia algorithm
      logical set
c        ... true if amrset has been called
c
      common /raset1/ u, c, cd, cm, i97, j97, set
c
c     INTERNAL:
c
      integer is1, is2
c        ... the two internal seeds used in Marsaglia algorithm
      integer IS1MAX
c        ... max value of first seed (is1), 31328
      integer IS2MAX
c        ... max value of second seed (is2), 30081
      integer i,j,k,l,m
c        ... used in generation of u()
      double precision s,t
c        ... used in generation of u()
      integer ii, jj
c        ... loop indices
c
      data IS1MAX, IS2MAX /31328, 30081/
c     data set /.false./    !now set in block data raset
c
c     --- construct two internal seeds from single unbound Amber seed ---
c
c         is1 and is2 are quotient and remainder of iseed/IS2MAX.  We add
c         one to keep zero and one results from both mapping to one.
c         max and min functions keep is1 and is2 in required bounds.
c
      is1 = max((iseed / IS2MAX)+1, 1)
      is1 = min(is1, IS1MAX)
c
      is2 = max(1, mod(iseed, IS2MAX)+1)
      is2 = min(is2, IS2MAX)
c
      i = mod(is1/177, 177) + 2
      j = mod(is1    , 177) + 2
      k = mod(is2/169, 178) + 1
      l = mod(is2    , 169)
c
      do 200 ii = 1, 97
           s = 0.0d0
           t = 0.5d0
           do 100 jj = 1, 24
                m = mod(mod(i*j, 179)*k, 179)
                i = j
                j = k
                k = m
                l = mod(53*l+1, 169)
                if (mod(l*m, 64) .ge. 32) s = s + t
                t = 0.5d0 * t
  100      continue
           u(ii) = s
  200 continue
c
      c  = 362436.0d0   / 16777216.0d0
      cd = 7654321.0d0  / 16777216.0d0
      cm = 16777213.0d0 / 16777216.0d0
c
      i97 = 97
      j97 = 33
c
      set = .true.
      return
      end
c-----------------------------------------------------------------------
      subroutine amrand(y)
C
C************************************************************************
C                              AMBER                                   **
C                                                                      **
C                  Copyright (c) 1986, 1991, 1995                      **
C             Regents of the University of California                  **
C                       All Rights Reserved.                           ** 
C                                                                      **
C  This software provided pursuant to a license agreement containing   **
C  restrictions on its disclosure, duplication, and use. This software **
C  contains confidential and proprietary information, and may not be   **
C  extracted or distributed, in whole or in part, for any purpose      **
C  whatsoever, without the express written permission of the authors.  **
C  This notice, and the associated author list, must be attached to    **
C  all copies, or extracts, of this software. Any additional           **
C  restrictions set forth in the license agreement also apply to this  **
C  software.                                                           **
c     Portable Random number generator by George Marsaglia
c     Amber 3.0 Rev A implementation by George Seibel
c
c     INPUT: (from common/raset1/)
c
      double precision u(97), c, cd, cm
c        ... real variables in Marsaglia algorithm
      integer i97, j97
c        ... pointers into u() in Marsaglia algorithm
      logical set
c        ... true if amrset has been called
c
      common /raset1/ u, c, cd, cm, i97, j97, set
c
c     OUTPUT:
c
c     y:  A random number between 0.0 and 1.0
c
#ifdef DPREC
      double precision y
#else
      real y
#endif
c
c     INTERNAL:
c
      double precision uni
c        ... working var. for random number
c
c     This random number generator originally appeared in *Toward a Universal
c     Random Number Generator* by George Marsaglia and Arif Zaman.  Florida
c     State University Report: FSU-SCRI-87-50 (1987)
c
c     It was later modified by F. James and published in *A Review of Pseudo-
c     random Number Generators*
c
c     This is claimed to be the best known random number generator available.
c     It passes ALL of the tests for random number generators and has a
c     period of 2^144, is completely portable (gives bit identical results on
c     all machines with at least 24-bit mantissas in the floating point
c     representation).
c
c     The algorithm is a combination of a Fibonacci sequence (with lags of 97
c     and 33, and operation "subtraction plus one, modulo one") and an
c     "arithmetic sequence" (using subtraction).
c
      if ( .not. set ) then
           write(6,'(a)') 'amrand not initd'
           call mexit(6, 1)
      endif
c
      uni = U(I97) - U(J97)
      if ( uni .lt. 0.0d0 ) uni = uni + 1.0d0
      U(I97) = uni
      I97 = I97 - 1
      if (I97 .eq. 0) I97 = 97
      J97 = J97 - 1
      if (J97 .eq. 0) J97 = 97
      C = C - CD
      if ( C .lt. 0.0d0 ) C = C + CM
      uni = uni - C
      if ( uni .lt. 0.0d0 ) uni = uni + 1.0d0
#ifdef DPREC
      y = uni
#else
      y = real(uni)
#endif
c
      return
      end
c-----------------------------------------------------------------------
      SUBROUTINE GAUSS(AM,SD,V)
C
C************************************************************************
C                              AMBER                                   **
C                                                                      **
C                  Copyright (c) 1986, 1991, 1995                      **
C             Regents of the University of California                  **
C                       All Rights Reserved.                           ** 
C                                                                      **
C  This software provided pursuant to a license agreement containing   **
C  restrictions on its disclosure, duplication, and use. This software **
C  contains confidential and proprietary information, and may not be   **
C  extracted or distributed, in whole or in part, for any purpose      **
C  whatsoever, without the express written permission of the authors.  **
C  This notice, and the associated author list, must be attached to    **
C  all copies, or extracts, of this software. Any additional           **
C  restrictions set forth in the license agreement also apply to this  **
C  software.                                                           **
#ifdef DPREC 
      implicit double precision (a-h,o-z)
#endif
C
c     This is a version of amrand() that adds the constraint of
c     a gaussian distribution. It also requires amrset to have
c     been called first, and "uses up" the same sequence that
c     amrand() does.
c
      double precision u(97), c, cd, cm
c        ... real variables in Marsaglia algorithm
      integer i97, j97
c        ... pointers into u() in Marsaglia algorithm
      logical set
c        ... true if amrset has been called
c
      common /raset1/ u, c, cd, cm, i97, j97, set
      data zero,six /0.0d0,6.0d0/
c
      if ( .not. set ) then
           write(6,'(a)') 'amrand not initd'
           call mexit(6, 1)
      endif
      A = zero
      DO 50 I = 1, 12
        uni = U(I97) - U(J97)
        if ( uni .lt. 0.0d0 ) uni = uni + 1.0d0
        U(I97) = uni
        I97 = I97 - 1
        if (I97 .eq. 0) I97 = 97
        J97 = J97 - 1
        if (J97 .eq. 0) J97 = 97
        C = C - CD
        if ( C .lt. 0.0d0 ) C = C + CM
        uni = uni - C
        if ( uni .lt. 0.0d0 ) uni = uni + 1.0d0
        A = A + uni
   50 continue
      V=(A-six)*SD+AM
      RETURN
      END
c-----------------------------------------------------------------------
