      program getcom
c**********************************************************************
c
c     GETCOM: A presumably operating system and machine independent 
c     program to grab SPASMS style meta-COMMON block information
c     from a routine to be used when unmasking split source files.
c     
c     SPASMS handles machine and operating system dependencies 
c     by commenting code with five character tokens which an
c     unmasking program (unmask) recognizes and processes (for
c     example, if a trigger is matched, particular code may be 
c     uncommented).  The unmask program also serves to distribute
c     COMMON block definitions to the various routines.  All of
c     the COMMON block information is defined in the main routine.
c
c     When using split sources, (i.e. the ``masked'' SPASMS src
c     which has been split up into a file per routine with
c     the fsplit utility) in order to distribute the COMMON block
c     info, the definitions from the main routine are needed.
c
c     So, this routine simply processes the main routine,
c     HARDWIRED to standard input, or unit 5, and outputs
c     to the file COMMON, all of the SPASMS-style COMMON 
c     block information necessary for unmasking and compiling
c     the split source files.
c
c     NOTE: This program is currently configured for use on source files
c     with no more than 80 characters per line, following the FORTRAN-77
c     (and SPASMS) convention of statements in columns 7-72.  
c     However, it can be easily modified to work with extended source
c     FORTRAN files, simply global replace 80 by 132 and 72 by 120.
c
c     BEWARE: Although the semantic content of the code is not altered
c     erroneous spaces (i.e. ' ') following a statement may be removed.
c     Additionally, on blank lines, spaces will be removed or added,
c     depending on the value of the IOUTSP parameter below.  However,
c     case will not be altered, and mixed case statements will be 
c     recognized.
c
c     ONE FINAL NOTE: This routine is hardcoded to read the input
c     source (main routine) from standard input, or unit 5, and to
c     output the results to a file named COMMON.
c     
c     Author: Thomas Cheatham, III.  March, 1993.
c
c**********************************************************************     
c
c     USER modifiable parameters
c
      parameter (ioutsp = 1)
c        ioutsp : Number of ``spaces'' to print out on blank lines,
c                 normally, zero or one.
      character*80 src, usrc
c        src    : where the text is read in, NOTE: hardcoded line length
c        usrc   : the upper case equivalent of the current src line
      character c
c
c**********************************************************************
c
c        Open COMMON
c
      open(11,file='COMMON',status='new',form='formatted')
c
c        Read in a line of test
 1    read(5, '(a80)', iostat=ierror, err=9005, end=9003) src
c     Search for cbeginblock lines; first screen for comment lines.
      c = src(1:1)
      if ( (c .ne. 'c') .and. (c .ne. 'C') ) goto 1000
c     Convert to upper case for test
      call ucaser (src, usrc)
      ipos = index( usrc(1:72), 'CBEGINBLOCK' )
      if (ipos .eq. 1) then
c        Found BEGINBLOCK line, write the line out
         iend = ieol(src)
         if (iend .eq. 0) iend = iend + ioutsp
         write(11,'(a)') src(1:iend)
c        Alternatively read/write lines until ENDBLOCK is found
 10      read(5, '(a80)', iostat=ierror, err=9005, end=9003) src
         iend = ieol(src)
         if (iend .eq. 0) iend = iend + ioutsp
         write(11,'(a)') src(1:iend)
         call ucaser (src, usrc)
         ipos = index( usrc(1:72), 'CENDBLOCK' )
         if (ipos .eq. 1) goto 1000
         goto 10
      endif
 1000 goto 1

c     jump over error handling
 9000 GOTO 9999

c     ERROR HANDLING

c     Handle input file end of file detection
 9003 goto 9999

c     Handle input file read error
 9005 print *, 'read error on unit 5, ierror = ', ierror
      stop 'ERROR'

 9999 END

c**********************************************************************
c     IEOL: function which returns the end of line of the text passed
c     into the routine (i.e. to rid trailing spaces, ' ')
c**********************************************************************
      integer function ieol(text)
      character*(*) text
      do 10 i= len(text), 1, -1
         if (text(i:i) .ne. ' ') goto 15
 10   continue
 15   ieol = i
      return
      end

c**********************************************************************
c     UCASER: subroutine to copy a character array (passed in the 
c     first parameter) to another (output) character array (the second 
c     parameter) converting the output array to UPPER case.
c**********************************************************************
      subroutine ucaser(intxt, outtxt)
      character*(*) intxt, outtxt
      character upper*26,lower*26
      data upper/'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      data lower/'abcdefghijklmnopqrstuvwxyz'/
      outtxt = intxt
      do 10 i=1,len(intxt)
      j = index(lower,intxt(i:i))
      if(j.ne.0) outtxt(i:i)=upper(j:j)
10    continue
      return
      end

