      program unmask
c*****************************************************************
c                        SPASMS                                 **
c                                                               **
c  Copyright (c) 1989 Regents of the University of California   **
c                  All Rights Reserved.                         **
c                                                               **
c    This software provided pursuant to a license agreement     **
c    containing restrictions on its disclosure, duplication,    **
c    and use.  This notice must be embedded in or attached      **
c    to all copies, including partial copies, of the software   **
c    or any revisions thereof.                                  **
c                                                               **
c*****************************************************************
c                                                               **
c    program unmask - generates machine specific fortran source **
c                     from the machine-independent source code  **
c                     file (SPASMS.SRC ---> FORTRAN).           **
c                                                               **
c    Authors : William C. Swope                                 **
c              David C. Spellmeyer                              **
c              Erik-Robert Evensen                              **
c                                                               **
c    This program was written very carefully to be machine and  **
c    operating system independent.                              **
c                                                               **
c    Modified by tec3 to read COMMON block information from     **
c    a file, COMMON, if it exists and can be opened to          **
c    facilitate split source compilation.  April, 1993.         **
c    Added HPUX token support                                   **
c                                                               **
c    Modified by Bill Ross to stop if unexpected option is      **
c    encountered in the CONFIG file.  May, 1993.                **
c                                                               **
c*****************************************************************
c
c     Program to activate the machine dependent or
c     operating system dependent lines of a fortran program.
c
c     First, a CONFIG file is opened if it exists and options
c     are read after the 1st line (a comment) is skipped.
c
c     Each line of an input source code file is then read in and
c     the first five characters are examined for a match with
c     one or more active trigger strings.  If no match is found, the
c     line is copied as is to an output source code file.
c     If a match is found, the trigger string is removed and
c     the modified line copied to the output source code file.
c     The trigger string is not case sensitive.
c     There are ntrig trigger strings.  The active ones are those
c     with a trigger string mask (tmask) value of true.
c     This mechanism allows multiple sets of lines to be unmasked.
c     For example, you may want to unmask lines with either cunix
c     or cvect, if you have a vector machine running unix.
c     Note that since the first five characters of each line are
c     examined for a match with the trigger string, lines with
c     trigger strings cannot be numbered.
c
c     This program also copies requested labelled common blocks from
c     the main program into the various subroutines.  The copy is
c     exact, which enforces consistent variable names and common block
c     lengths.  All labelled common blocks must be declared in the
c     main program.  It is strongly suggested that this be the only way
c     common blocks be used in SPASMS in order to reduce programming
c     errors.  This feature of unmask serves a function usually served
c     by "INCLUDE" compiler directives.  However, since INCLUDE is an
c     extension to the FORTRAN-77 standard, it tends to be FORTRAN
c     implementation dependent.  It is also operating system dependent
c     since it involves file names.
c
c
      parameter (mxcom=200)
      common/cinclud/comblk,comlab
      common/iinclud/nline,nchar,ncoms,npars,icomfl
      character*72 comblk(100,mxcom)
      character*6  comlab(mxcom)
      dimension nline(mxcom),nchar(mxcom)
      
      character*72 inline
      character*5  field,fmt
      character*5  trig(100)
      character*6  opt
      logical tmask(100)
      data ntrig/14/
      data trig(1)/'cunix'/, tmask(1)/.false./,
     x     trig(2)/'cvect'/, tmask(2)/.false./,
     x     trig(3)/'cscal'/, tmask(3)/.false./,
     x     trig(4)/'ci390'/, tmask(4)/.false./,
     x     trig(5)/'cn390'/, tmask(5)/.false./,
     x     trig(6)/'crs6k'/, tmask(6)/.false./,
     x     trig(7)/'cf264'/, tmask(7)/.false./,
     x     trig(8)/'csmlm'/, tmask(8)/.false./,
     x     trig(9)/'clrgm'/, tmask(9)/.false./,
     x    trig(10)/'ccray'/,tmask(10)/.false./,
     x    trig(11)/'cvms '/,tmask(11)/.false./,
     x    trig(12)/'cmmwb'/,tmask(12)/.false./,
     x    trig(13)/'cdebg'/,tmask(13)/.false./,
     x    trig(14)/'chpux'/,tmask(14)/.false./
c
c     read the configuration file built by the shell
c
      open (12, file = 'CONFIG', status = 'unknown', iostat = ierr)
      if (ierr.eq.0) then
         read(12, '(a)', end = 9) opt
    8    continue
            read(12, '(a)', end = 9) opt
            call ucase(opt)
            if (opt.eq.'UNIX  ') then
                       tmask( 1) = .true.
            else if (opt.eq.'VECTOR') then
                       tmask( 2) = .true.
            else if (opt.eq.'SCALAR') then
                       tmask( 3) = .true.
            else if (opt.eq.'IBM390') then
                       tmask( 4) = .true.
            else if (opt.eq.'I390  ') then
                       tmask( 4) = .true.
            else if (opt.eq.'N390  ') then
                       tmask( 5) = .true.
            else if (opt.eq.'RS6000') then
                       tmask( 6) = .true.
            else if (opt.eq.'FPS264') then
                       tmask( 7) = .true.
            else if (opt.eq.'SMALL ') then
                       tmask( 8) = .true.
            else if (opt.eq.'LARGE ') then
                       tmask( 9) = .true.
            else if (opt.eq.'CRAY  ') then
                       tmask(10) = .true.
            else if (opt.eq.'VMS   ') then
                       tmask(11) = .true.
            else if (opt.eq.'MMWB  ') then
                       tmask(12) = .true.
            else if (opt.eq.'DEBUG ') then
                       tmask(13) = .true.
            else if (opt.eq.'HPUX  ') then
                       tmask(14) = .true.
            else
               write(*,'(a,a)')
     .            ' Error:  unknown CONFIG option: ', opt
               stop ': bad CONFIG file'
            endif
         go to 8
    9    continue
      endif
c
c     check for reasonable switch settings and stop if not!
c
c either vector or scalar should be set, not both
      if(tmask(2).and.tmask(3)) then
         write(*,'(a,a)')
     .      ' Error:  Both scalar and vector lines are',
     .      ' activated.  Select one or the other.'
         stop
      endif
c if neither is set, assume scalar
      if(.not.tmask(2).and..not.tmask(3)) then
         tmask(3) = .true.
      endif
c either i390 or n390 should be set, not both
      if(tmask(4).and.tmask(5)) then
         write(*,'(a,a)')
     .      ' Error:  Both IBM S/390 and non IBM S/390 lines',
     .      ' are activated.  Select one or the other.'
         stop
      endif
c if neither is set, assume n390
      if(.not.tmask(4).and..not.tmask(5)) then
         tmask(5) = .true.
      endif
c either small memory or large memory should be set, not both
      if(tmask(8).and.tmask(9)) then
         write(*,'(a,a,/,a)')
     .      ' Error:  Both small memory and large',
     .      ' memory lines are are activated.',
     .      ' Select one or the other.'
         stop
      endif
c if neither is set, assume small
      if(.not.tmask(8).and..not.tmask(9)) then
         tmask(8) = .true.
      endif
 
c
c     upper case the trigger strings
c
      do 10 i=1,ntrig
10    call ucase(trig(i))
c
c     open files
c
      open(10,file='INPUT', status='old',form='formatted')
      open(11,file='OUTPUT',status='new',form='formatted')
      open(12,file='COMMON',err=97,status='old',form='formatted')
      
      icomfl = 12 
      goto 99
 97   icomfl = 10
 99   continue
c
c     get and save all the common blocks in the main program
c
      call getcom
c
c     check all of the common blocks against the triggers
c
      do 120 i = 1 , ncoms
         nln = nline(i)
         do 130 j = 1 , nln
            inline = comblk(j,i)
            field  = inline
            call ucase(field)
            do 140 k=1,ntrig
               if(.not.tmask(k)) go to 140
               if(field.eq.trig(k)) then
                  call ccount(inline,icnt)
                  inline='     '//inline(6:icnt)
                  comblk(j,i)=inline
               endif
140         continue
130      continue
120   continue
c
c     at this point, the common blocks should have been unmasked, and
c     the input file should have been re-wound.
c
20    read(10,'(a72)',end=9999) inline
      field=inline
      call ucase(field)
c
c     check field against trigger strings for a match
c
      do 30 i=1,ntrig
      if(.not.tmask(i)) go to 30
      if(field.eq.trig(i)) go to 40
30    continue
c     no match found - just copy it to output file
c     following catches IBM compiler directives of form '@PROCESS'
      if(tmask(4).and.field.eq.'C@PRO') inline=inline(2:)
      go to 100
c     match found - clear the field and copy it to output file
40    inline='     '//inline(6:)
100   call ccount(inline,icnt)
      write(fmt,'(''(a'',i2,'')'')') icnt
      write(11,fmt) inline(1:icnt)
c     following looks for include string
      if(field.eq.'CINCL') call putcom(inline)
      go to 20
c
9999  close(10)
      close(11)
      end
c
c     subroutine to examine inline string from end until it finds a
c     nonblank character.  icnt is the index of the last nonblank.
c
      subroutine ccount(inline,icnt)
      character*(*) inline
      l=len(inline)
      do 10 i=l,1,-1
      icnt=i
      if(inline(i:i).ne.' ') go to 20
10    continue
20    return
      end
      subroutine ucase(field)
c*****************************************************************
c                                                               **
c    Subroutine : ucase - sets characters of a string to upper  **
c    Author  : William Swope                                    **
c                                                               **
c*****************************************************************
c
c This routine is written to be FORTRAN-implementation independent
c and will work on machines using EBCDIC as well as ASCII character
c representation.
c
      character*(*) field
      character upper*26,lower*26
      data upper/'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      data lower/'abcdefghijklmnopqrstuvwxyz'/
      do 10 i=1,len(field)
      j = index(lower,field(i:i))
      if(j.ne.0) field(i:i)=upper(j:j)
10    continue
      return
      end
c
c     subroutine to read through the main program to save all
c     labelled blocks of code.  Blocks must begin with a
c     cbeginblock/blknam/     line and end with a
c     cendblock               line.
c
      subroutine getcom
c
c     mxcom is the maximum number of labelled blocks
      parameter (mxcom=200)
      common/cinclud/comblk,comlab
      common/iinclud/nline,nchar,ncoms,npars,icomfl
c
c     comblk is array of labelled blocks.  Note: the FORTRAN-77
c     standard allows a statement to have 19 continuation lines,
c     so the maximum number of lines per single statement block
c     should be 20 or less.  Since labelled blocks may contain
c     more than one statement and since we are copying comments,
c     too, we are allowing up to 100 lines per block.
c
      character*72 comblk(100,mxcom), line, string, line2
c
c     comlab is the (up to) six-character label of the labelled block
      character*6  comlab(mxcom)
c
c     nline is the number of lines in the common block,
c     nchar the length of the name of the common block
      dimension nline(mxcom),nchar(mxcom)
c
c     ncoms is the number of labelled blocks found
c
      ncoms = 0
c
      rewind icomfl
20    read(icomfl,'(a72)',end=60) line
c
c     look for end of main program
c     ...only necessary if getting common blocks from INPUT, i.e. unit 10
c
      if ( icomfl .eq. 10 ) then
         if(line(1:6).eq.'      '.and.
     .        (index(line(6:),' end    ').ne.0 .or.
     .        index(line(6:),' END    ').ne.0 .or.
     .        index(line(6:),' End    ').ne.0     )) go to 60
      endif

      if(line(1:1).ne.'c'.and.line(1:1).ne.'C') go to 50
c
c     Here if still in main and line is a comment.
c     Look for a "begin block" statement with two slashes.
c
      call cmprs(line,line2,nch)
      if(line2(1:11).ne.'CBEGINBLOCK') go to 50
      isl1 = 1+index(line,'/')
      isl2 = isl1+index(line((isl1+1):),'/')-1
      if(isl1.eq.1 .or. isl2.le.isl1) go to 50
      if(ncoms+1.gt.mxcom) then
        write(*,'(a,a)')
     .     ' Error:  Too many labelled blocks for',
     .     ' unmask.  Increase size of mxcom in unmask.'
        stop
      endif
      ncoms=ncoms+1
c
c     Here when a block has been found.  Extract and save name of
c     labelled block in line(isl1:isl2).
c
      call cmprs(line(isl1:isl2),string,nlong)
      if(nlong.le.0 .or. nlong.gt.6 ) then
        write(*,'(a,/,a,/,a,/,a,a)')
     .     ' Error:  Labelled block name greater than six characters.',
     .     ' FORTRAN standard limits labels to six,',
     .     ' and we are following the same convention here.',
     .     ' Offending label is: ', string(1:nlong)
        stop
      endif
      comlab(ncoms)=string(1:nlong)
      nchar(ncoms)=nlong
      comblk(1,ncoms)=line
      nline(ncoms)=1
c
c     Read remaining lines of the labelled block until there
c     are no more.
c
30    read(icomfl,'(a72)',end=60) line
      if(nline(ncoms)+1.gt.100) then
        write(*,'(a,a,/,a,a)')
     .     ' Error:  Labelled block is too long.',
     .     ' Current limits is 100 lines.',
     .     ' Offending block is: ', comlab(ncoms)
        stop
      endif
      nline(ncoms)=nline(ncoms)+1
      comblk(nline(ncoms),ncoms)=line
      call cmprs(line,line2,nch)
      if(line2(1:9).eq.'CENDBLOCK') go to 50
      go to 30
c
c     Here when done reading in a labelled block
c
50    continue
      go to 20
c     ...only really need to rewind if icomfl .eq. 10
60    rewind icomfl
c
c     At this point all the labelled blocks have been found and
c     saved.  There are ncoms of them, and the ith one has
c     nline(i) lines, stored in comblk(1-nline(i),i), and the label
c     is nchar(i) characters stored in comlab(i).
c
c     write(*,'(/,'' number of saved labelled blocks: '',i5)') ncoms
c     do 100 i=1,ncoms
c       write(*,'(/,'' i='',i3,'' label=*'',a,''*'')')
c    .                     i,comlab(i)(1:nchar(i))
c       do 70 j=1,nline(i)
c         write(*,'('' j='',i2,'' *'',a,''* '')') j,comblk(j,i)(1:65)
70      continue
100   continue
 
      return
      end
c----------------------------------------------------------------
c     subroutine to squeeze out blanks, left justify, capitalize,
c     and count chars
c
      subroutine cmprs(strin,strout,nch)
      character*(*) strin, strout
      nch = 0
      do 10 i = 1,len(strin)
      if(strin(i:i).eq.' ') go to 10
      nch = nch+1
      strout(nch:nch) = strin(i:i)
10    continue
      call ucase(strout(1:nch))
      end
c
c     subroutine to copy the labelled block specified in the
c     'CINCLUDE' line of the source code file.  This routine is
c     the complement of getcom.
c
      subroutine putcom(line)
      parameter (mxcom=200)
      common/cinclud/comblk,comlab
      common/iinclud/nline,nchar,ncoms,npars,icomfl
      character*72 comblk(100,mxcom)
      character*6  comlab(mxcom)
      dimension nline(mxcom),nchar(mxcom)
      character*(*) line
      character*72 string
      character*5  fmt
c
c     First, extract name of labelled common from between slashes.
c
      isl1 = 1+index(line,'/')
      isl2 = isl1+index(line((isl1+1):),'/')-1
      if(isl1.eq.0 .or. isl2.le.isl1) then
        write(*,'(a,a,/,a,a)')
     .     ' Error:  Block name to be included must',
     .     ' be specified between slashes.',
     .     ' Line found: ', line(1:40)
        goto 21
      endif
      call cmprs(line(isl1:isl2),string,nlong)
      if(nlong.le.0 .or. nlong.gt.6 ) then
        write(*,'(a,/,a,a)')
     .     ' Error:  Labelled block name greater than six characters.',
     .     ' Offending label is: ', string(1:nlong)
        goto 21
      endif
c
c     search for desired block name among those saved
c
      do 20 i=1,ncoms
        if(string(1:nlong).eq.comlab(i)(1:nchar(i))) go to 30
20    continue
c
c     here if none found
c
      write(*,'(a,/,a,a)')
     .   ' Error: sought-for block does not exist.',
     .   ' Desired labelled blok is ', string(1:nlong)
21    continue
      write(*,'('' Writing syntax error to draw compiler attention '')')
      write(11, '(''      do: fix unmask error.'')')
      stop 'ERROR'
c
c     here if found, it is common block i
c
30    continue
      do 40 j=1,nline(i)
        call ccount(comblk(j,i),icnt)
        write(fmt,'(''(a'',i2,'')'')') icnt
        write(11,fmt) comblk(j,i)(1:icnt)
40    continue
 
100   return
      end
